sets
i /1*6/
km /1*5/
Zs /1*4/
kC /1*6/
;

alias(i,j);

scalars
Tsoil 'Soil temperature [R]' /519.67/
R 'gas constant [lbf-ft/(lbm-R)]' /85.2/ 
L 'Pipe Length [mile]'  /50/
d 'Pipe Diameter [ft]' /3/
Z_st 'Compressability Factor at STP' /1/
conv1 'lb/min to kg/s' /0.00755987/
conv2 'kg^2/s^2 to lb^2/min^2' /17497.204729/
conv3 '[miles-lbm-lbf/(ft^4-min^2)] to [psi^2]' /2.1983696645E-6/
conv4 'psi to MPa' /0.00689476/
conv5 'psi to lbf/ft^2' /144/
Z 'Constant Compressibility Factor' /0.95/
kappa 'Isentropic Exponent' /1.287/
S_g 'Specific Gravity' /0.6248/
ff 'Friction Factor' /0.0085/
A6 'fc para 1' /0.0266/
B6 'fc para 2' /38.1969/
C6 'fc para 3' /-3.4865/
D6 'fc para 4' /2.3791/
E6 'fc para 5' /439.7503/
F6 'fc para 6' /-460.6632/
;

parameters
arcs_p(i,j) 'set of passive arcs',
arcs_a(i,j) 'set of active arcs',
Vext(i) 'external node V flow [MMSCFD]',
qext(i) 'external node m flow [lbm/min]',
Plow 'lower limit node Pressure [psi]',
Phigh 'upper limit node Pressure [psi]',
T(i) 'node Temperature [K]'
mappa 'Isentropic exponent term [1]'
K_p 'Pipeline Parameter 1' 
c_p 'Pipeline Parameter 2'
aC(kC) 'Wu Comp D Para 1' 
bC(kC) 'Wu Comp D Para 2'
;

K_p = 1.3305E5*Z*S_g*Tsoil;

c_p = K_p*ff*L/(d**5);

mappa = (kappa - 1)/kappa;

arcs_p('1','2') = 1; arcs_a('2','3') = 1; arcs_p('3','4') = 1; arcs_a('4','5') = 1; arcs_p('5','6') = 1;

Vext('1') = 600; Vext('6') = -600;
qext(i) = Vext(i)*1E6*14.6959/Z_st/R/Tsoil/24/60*conv5;

aC('1') = 0.0185; aC('2') = 0; aC('3') = -0.0328; aC('4') = -0.0047; aC('5') = 0; aC('6') = 0.0096;
bC('1') = 0.6778; bC('2') = 1.4729; bC('3') = 3.6853; bC('4') = 1.2332; bC('5') = 1.0620; bC('6') = 0.4921;

T(i) = Tsoil;

Plow = 600;
Phigh = 800;

variables
phi 'fuel cost [10^4 lbf-ft/min]';

positive variables
q(i,j) 'arc flow rate [lbm/min]'
P(i) 'Node Pressure [psi]'
g(i,j) 'Fuel Cost [$]'
Prat(i,j) 'Compressor Pressure ratio [1]'
zeta(i,j) 'Lumped friction factor and squared flow rate [kg^2/s^2]'
P_m(i,j) 'Average arc Pressure [psi]'
qij_Pi(i,j) 'ratio of mass flow over suction pressure' 
Pj_Pi(i,j) 'ratio of discharge pressure over suction pressure' 
;

integer variables
ns(i,j) 'number of compressor units active'
;

equations
FC
mass_bal(i)
piperes(i,j)
RHS_pres(i,j)
CompD1(i,j)
CompD2(i,j)
CompD3(i,j)
CompD4(i,j)
CompD5(i,j)
CompD6(i,j)
cost(i,j)
rat1(i,j)
rat2(i,j)
;

FC.. phi =e= sum((i,j) $(arcs_a(i,j) = 1), g(i,j)*q(i,j))/1E4;

cost(i,j) $(arcs_a(i,j) = 1).. g(i,j) =e= (A6*qij_Pi(i,j)**2 + B6*Pj_Pi(i,j)**2 + C6*qij_Pi(i,j)*Pj_Pi(i,j) + D6*qij_Pi(i,j) + E6*Pj_Pi(i,j) + F6);

rat1(i,j) $(arcs_a(i,j) = 1).. qij_Pi(i,j)*P(i) =e= q(i,j);

rat2(i,j) $(arcs_a(i,j) = 1).. Pj_Pi(i,j)*P(i) =e= P(j);

mass_bal(i).. sum(j $((arcs_a(j,i) = 1) or arcs_p(j,i) = 1), q(j,i)) - sum(j $((arcs_a(i,j) = 1) or (arcs_p(i,j) = 1)), q(i,j)) =e= -qext(i);

piperes(i,j) $(arcs_p(i,j) = 1).. (P(i)**2 - P(j)**2) =e= (L*R*Z*Tsoil/((pi*(d/2)**2)**2*d)*zeta(i,j))*conv3;

RHS_pres(i,j) $(arcs_p(i,j) = 1).. zeta(i,j) =e= ff*q(i,j)**2;

CompD1(i,j) $(arcs_a(i,j) = 1).. P(j) =l= aC('1')*q(i,j) + bC('1')*P(i);
CompD2(i,j) $(arcs_a(i,j) = 1).. P(j) =l= bC('2')*P(i);
CompD3(i,j) $(arcs_a(i,j) = 1).. P(j)*ns(i,j) =l= aC('3')*q(i,j) + bC('3')*P(i)*ns(i,j);
CompD4(i,j) $(arcs_a(i,j) = 1).. P(j) =g= aC('4')*q(i,j) + bC('4')*P(i);
CompD5(i,j) $(arcs_a(i,j) = 1).. P(j) =g= bC('5')*P(i);
CompD6(i,j) $(arcs_a(i,j) = 1).. P(j)*ns(i,j) =g= aC('6')*q(i,j) + bC('6')*P(i)*ns(i,j);

P.lo(i) = Plow;
P.up(i) = Phigh;
q.lo(i,j) = 0;
q.l(i,j) = 25000;
ns.up(i,j) = 5;
ns.l(i,j) = 1;

q.scale(i,j) = 1E3;
P.scale(i) = 1E1;
g.scale(i,j) = 1;
Prat.scale(i,j) = 1E-1;
zeta.scale(i,j) = 1E5;
P_m.scale(i,j) = 1E1;
qij_Pi.scale(i,j) = 1;
Pj_Pi.scale(i,j) = 1E-1;

option minlp = scip;
option reslim = 10000000;

model FCMP_S /
all
/;

option optcr = 1e-5;
option optca = 1e-5;

FCMP_S.scaleopt = 1;

solve FCMP_S using minlp minimizing phi;


display q.L, P.l, ns.l, g.l, zeta.l, qij_Pi.l, Pj_Pi.l;
