sets
i /1*18/
km /1*4/
Zs /1*2/
kH_l /1*4/
kH_u /1*4/
kH_srg /1*3/
kH_stw /1*3/
kg /1*4/
;

alias(i,j);

scalars
Tsoil 'Soil temperature [R]' /519.67/
R 'gas constant [lbf-ft/(lbm-R)]' /85.2/ 
L 'Pipe Length [mile]'  /50/
d 'Pipe Diameter [ft]' /3/
Z_st 'Compressability Factor at STP' /1/
conv1 'lb/min to kg/s' /0.00755987/
conv2 'kg^2/s^2 to lb^2/min^2' /17497.204729/
conv3 '[miles-lbm-lbf/(ft^4-min^2)] to [psi^2]' /2.1983696645E-6/
conv4 'psi to MPa' /0.00689476/
conv5 'psi to lbf/ft^2' /144/
conv6 'R to K' /0.55555556/
M1 'Big M for Mappa' /1/
M2 'Big M for Z' /1/
M3 'Big M for H_l' /1.3E4/
M4 'Big M for H_srg' /1E4/
;

parameters
arcs_p(i,j) 'set of passive arcs',
arcs_a(i,j) 'set of active arcs',
Vext(i) 'external node V flow [MMSCFD]',
qext(i) 'external node m flow [lbm/min]',
Plow(i) 'lower limit node Pressure [psi]',
Phigh(i) 'upper limit node Pressure [psi]',
T(i) 'node Temperature [K]'
dk(Zs) 'Z reg coef 2 [1]'
ck(Zs) 'Z reg coef 1 [MPa^-1]'
akm(km) 'kappa reg coef 1 [K^-1]'
bkm(km) 'kappa reg coef 2 [MPa^-1]'
ckm(km) 'kappa reg coef 3 [1]'
ckH_l(kH_l) 'H_l coef 1 [min/ft^3 1x10^3-lbf-ft/lbm]'
dkH_l(kH_l) 'H_l coef 2 [1x10^3-lbf-ft/lbm]'
ckH_u(kH_u) 'H_u coef 1 [min/ft^3 1x10^3-lbf-ft/lbm]'
dkH_u(kH_u) 'H_l coef 2 [1x10^3-lbf-ft/lbm]'
ckH_srg(kH_srg) 'H_srg coef 1 [min/ft^3 1x10^3-lbf-ft/lbm]'
dkH_srg(kH_srg) 'H_l coef 2 [1x10^3-lbf-ft/lbm]'
ckH_stw(kH_stw) 'H_stw coef 1 [min/ft^3 1x10^3-lbf-ft/lbm]'
dkH_stw(kH_stw) 'H_l coef 2 [1x10^3-lbf-ft/lbm]'
akg(kg) 'gc coef 1 [min/ft^3 1x10^3-lbf-ft/lbm]'
bkg(kg) 'gc coef 2 [1]'
ckg(kg) 'gc coef 3 [1x10^3-lbf-ft/lbm]'
;

arcs_a('1','2') = 1; arcs_p('2','3') = 1; arcs_a('3','4') = 1; arcs_p('4','5') = 1; arcs_p('5','6') = 1;
arcs_p('5','7') = 1; arcs_a('3','8') = 1; arcs_p('8','9') = 1; arcs_p('9','10') = 1;
arcs_a('7','11') = 1; arcs_p('11','12') = 1; arcs_p('12','13') = 1; arcs_p('12','14') = 1;
arcs_a('14','15') = 1; arcs_p('15','16') = 1; arcs_p('15','17') = 1; arcs_p('17','18') = 1;

Vext('1') = 800; Vext('5') = -100; Vext('9') = -100; Vext('6') = -150; Vext('7') = 500; Vext('10') = -300; Vext('13') = -400; 
Vext('14') = 1000; Vext('16') = -625; Vext('17') = -500; Vext('18') = -125;

qext(i) = Vext(i)*1E6*14.6959/Z_st/R/Tsoil/24/60*conv5;

ck('1') = -25.136118/1000; dk('1') = 0.998755;
*ck('2') = -20.582142/1000; dk('2') = 0.965961;

akm('1') = -0.196274E-3; akm('2') = -0.881420E-3; akm('3') = -2.281592E-3; akm('4') = -0.285367E-3;
bkm('1') = 2.134555E-3; bkm('2') = 20.995274E-3; bkm('3') = 34.205039E-3; bkm('4') = 10.340562E-3;
ckm('1') = 0.276919; ckm('2') = 0.377140; ckm('3') = 0.660269; ckm('4') = 0.273620;

ckH_l('1') = -0.223675E-3; ckH_l('2') = -0.524862E-3; ckH_l('3') = -0.894200E-3; ckH_l('4') = -1.262928E-3;
dkH_l('1') = 6.490016; dkH_l('2') = 9.257721; dkH_l('3') = 13.047961; dkH_l('4') = 17.130303;

ckH_u('1') = -2.369797E-3; ckH_u('2') = -0.371089E-3; ckH_u('3') = -1.617374E-3; ckH_u('4') = -0.927015E-3;
dkH_u('1') = 60.362712; dkH_u('2') = 22.026407; dkH_u('3') = 44.770128; dkH_u('4') = 31.500468;

ckH_srg('1') = 1.538237E-3; ckH_srg('2') = 1.899720E-3; ckH_srg('3') = 2.344936E-3;
dkH_srg('1') = -5.946145; dkH_srg('2') = -9.069204; dkH_srg('3') = -13.818207;

ckH_stw('1') = 0.55099E-3; ckH_stw('2') = 0.682284E-3; ckH_stw('3') = 0.441595E-3;
dkH_stw('1') = -4.426806; dkH_stw('2') = -6.783055; dkH_stw('3') = -2.839851;

akg('1') = -0.938222E-6; akg('2') = 4.531850E-6; akg('3') = 3.199214E-6; akg('4') = 1.696372E-6;
bkg('1') = 0.012282; bkg('2') = 0.009219; bkg('3') =  0.009243; bkg('4') =  0.010406;
ckg('1') = 0.004784; ckg('2') = -0.041031; ckg('3') = -0.020461; ckg('4') = -0.009882;

T(i) = Tsoil;

Plow('1') = 600; Plow('2') = 600; Plow('3') = 450; Plow('5') = 450; Plow('6') = 450; Plow('7') = 450; Plow('9') = 450; Plow('4') = 500; Plow('10') = 400; Plow('8') = 550; Plow('11') = 50;  Plow('12') = 50; Plow('13') = 50; Plow('14') = 50; Plow('15') = 100; Plow('16') = 100; Plow('17') = 100; Plow('18') = 100;
Phigh(i) = 800; Phigh('1') = 700; 

variables
phi 'fuel cost [10^4 lbf-ft/min]'
H_l_lim(i,j)
H_srg_lim(i,j);

positive variables
q(i,j) 'arc flow rate [lbm/min]'
V(i,j) 'volumetric flow rate [ft^3/min]'
Had(i,j) 'Compressor Head [lbf-ft/lbm]'
P(i) 'Node Pressure [psi]'
g(i,j) 'Fuel Cost [$]'
Prat(i,j) 'Compressor Pressure ratio [1]'
zeta(i,j) 'Lumped friction factor and squared flow rate [kg^2/s^2]'
Z_m(i,j) 'Average arc Compressability factor [1]'
P_m(i,j) 'Average arc Pressure [psi]'
z(i) 'Node Compressability Factor [1]'
m(i,j) 'Fuction of isentropic exponent [1]'
;

binary variable
b_Zi(i,j,Zs)
b_Zm(i,j,Zs)
b_m(i,j,km)
b_H_l(i,j,kH_l)
b_H_srg(i,j,kH_srg)
;

integer variables
ns(i,j) 'number of compressor units active'
;

equations
FC
mass_bal(i)
piperes(i,j)
RHS_pres(i,j)
Qflow(i,j)
HadCalc(i,j)
Pratio(i,j)
cost(i,j,kg)
Pmean(i,j)
Zm_l(i,j,Zs)
Zm_h(i,j,Zs)
mut_Zm(i,j)
Zi_l(i,j,Zs)
Zi_h(i,j,Zs)
mut_Zi(i,j)
mappa_l(i,j,km)
mappa_u(i,j,km)
mut_m(i,j)
H_stw(i,j,kH_stw)
H_u(i,j,kH_u)
H_l_l(i,j,kH_l)
H_l_h(i,j,kH_l)
mut_H_l(i,j)
H_srg_l(i,j,kH_srg)
H_srg_h(i,j,kH_srg)
mut_H_srg(i,j)
H_srg_con(i,j)
H_l_con(i,j)
;
  
FC.. phi =e= sum((i,j) $(arcs_a(i,j) = 1), g(i,j)*q(i,j))/1E4;

cost(i,j,kg) $(arcs_a(i,j) = 1).. g(i,j) =g= (akg(kg)*V(i,j) + bkg(kg)*Had(i,j)/1E3 + ckg(kg))*1E3;

mass_bal(i).. sum(j $((arcs_a(j,i) = 1) or arcs_p(j,i) = 1), q(j,i)) - sum(j $((arcs_a(i,j) = 1) or (arcs_p(i,j) = 1)), q(i,j)) =e= -qext(i);

piperes(i,j) $(arcs_p(i,j) = 1).. (P(i)**2 - P(j)**2) =e= (L*R*Z_m(i,j)*T(i)/((pi*(d/2)**2)**2*d)*(zeta(i,j)*conv2))*conv3;

RHS_pres(i,j) $(arcs_p(i,j) = 1).. zeta(i,j) =e= (1.071003930/100)*(q(i,j)*conv1)**2 + 0.02370952*q(i,j)*conv1;

Pmean(i,j) $(arcs_p(i,j) = 1).. P_m(i,j) =e= 0.5*(P(i) + P(j));

Zm_l(i,j,Zs) $(arcs_p(i,j) = 1).. Z_m(i,j) =g= ck(Zs)*(P_m(i,j)*conv4) + dk(Zs);

Zm_h(i,j,Zs) $(arcs_p(i,j) = 1).. Z_m(i,j) =l= ck(Zs)*(P_m(i,j)*conv4) + dk(Zs) + M2*(1 - b_Zm(i,j,Zs));

mut_Zm(i,j) $(arcs_p(i,j) = 1).. sum(Zs, b_Zm(i,j,Zs)) =e= 1;

Qflow(i,j) $(arcs_a(i,j) = 1).. V(i,j)*P(i)*ns(i,j)*conv5 =e= z(i)*R*T(i)*q(i,j);

Zi_l(i,j,Zs) $(arcs_a(i,j) = 1).. z(i) =g= ck(Zs)*(P(i)*conv4) + dk(Zs);

Zi_h(i,j,Zs) $(arcs_a(i,j) = 1).. z(i) =l= ck(Zs)*(P(i)*conv4) + dk(Zs) + M2*(1 - b_Zi(i,j,Zs));

mut_Zi(i,j) $(arcs_a(i,j) = 1).. sum(Zs, b_Zi(i,j,Zs)) =e= 1;

HadCalc(i,j) $(arcs_a(i,j) = 1).. Had(i,j)*m(i,j) =e= (z(i)*R*T(i)*(Prat(i,j)**(m(i,j))-1));

mappa_l(i,j,km) $(arcs_a(i,j) = 1).. m(i,j) =g= akm(km)*(T(i)*conv6) + bkm(km)*(P(i)*conv4) + ckm(km);

mappa_u(i,j,km) $(arcs_a(i,j) = 1).. m(i,j) =l= akm(km)*(T(i)*conv6) + bkm(km)*(P(i)*conv4) + ckm(km) + M1*(1 - b_m(i,j,km));

mut_m(i,j)  $(arcs_a(i,j) = 1).. sum(km, b_m(i,j,km)) =e= 1;

Pratio(i,j)  $(arcs_a(i,j) = 1).. Prat(i,j)*P(i) =e= P(j);

H_stw(i,j,kH_stw) $(arcs_a(i,j) = 1).. Had(i,j) =g= (ckH_stw(kH_stw)*V(i,j) + dkH_stw(kH_stw))*1E3;

H_u(i,j,kH_u) $(arcs_a(i,j) = 1).. Had(i,j) =l= (ckH_u(kH_u)*V(i,j) + dkH_u(kH_u))*1E3;

H_l_l(i,j,kH_l) $(arcs_a(i,j) = 1).. H_l_lim(i,j) =g= (ckH_l(kH_l)*V(i,j) + dkH_l(kH_l))*1E3 - M3*(1 - b_H_l(i,j,kH_l));

H_l_h(i,j,kH_l) $(arcs_a(i,j) = 1).. H_l_lim(i,j) =l= (ckH_l(kH_l)*V(i,j) + dkH_l(kH_l))*1E3;

mut_H_l(i,j) $(arcs_a(i,j) = 1).. sum(kH_l, b_H_l(i,j,kH_l)) =e= 1;

H_srg_l(i,j,kH_srg) $(arcs_a(i,j) = 1).. H_srg_lim(i,j) =g= (ckH_srg(kH_srg)*V(i,j) + dkH_srg(kH_srg))*1E3;

H_srg_h(i,j,kH_srg) $(arcs_a(i,j) = 1).. H_srg_lim(i,j) =l= (ckH_srg(kH_srg)*V(i,j) + dkH_srg(kH_srg))*1E3 + M4*(1 - b_H_srg(i,j,kH_srg));

mut_H_srg(i,j) $(arcs_a(i,j) = 1)..  sum(kH_srg, b_H_srg(i,j,kH_srg)) =e= 1;

H_srg_con(i,j) $(arcs_a(i,j) = 1).. Had(i,j) =g= H_l_lim(i,j);

H_l_con(i,j) $(arcs_a(i,j) = 1).. Had(i,j) =l= H_srg_lim(i,j);

P.lo(i) = Plow(i);
P.up(i) = Phigh(i);
q.lo(i,j) = 0;
q.l(i,j) $(arcs_p(i,j) = 1 or arcs_a(i,j) = 1) = 25000;
V.lo(i,j) = 0.7E4;
V.up(i,j) = 2.2E4;
ns.up(i,j) = 5;
ns.l(i,j) $(arcs_a(i,j) = 1) = 1;

q.scale(i,j) = 1E3;
V.scale(i,j) = 1E2;
Had.scale(i,j) = 1E2;
P.scale(i) = 1E1;
g.scale(i,j) = 1E1;
Prat.scale(i,j) = 1E-1;
zeta.scale(i,j) = 1E1;
Z_m.scale(i,j) = 1E-2;
P_m.scale(i,j) = 1E1;
z.scale(i) = 1E-2;
m.scale(i,j) = 1E-2;
phi.scale = 1E1;

option minlp = scip;
option reslim = 10000000;

model FCMP_N /
all
/;

option optcr = 1e-5;
option optca = 1e-5;

FCMP_N.scaleopt = 1;
FCMP_N.OptFile = 1;

solve FCMP_N using minlp minimizing phi;

display q.l, Had.l,P.l, z.l, Z_m.l, V.l, m.l, zeta.l, ns.l, g.l;
