sets
i /1*18/
km /1*4/
Zs /1*2/
;

alias(i,j);

scalars
Tsoil 'Soil temperature [R]' /519.67/
R 'gas constant [lbf-ft/(lbm-R)]' /85.2/ 
Ah 'AdHead Parameter [lbf-ft/(lbm-rpm)]' /0.6824E-3/
Bh 'AdHead Parameter [lbf-ft-min/(lbm-ft^3-rpm)]' /-0.9002E-3/
Ch 'AdHead Parameter [lbf-ft/(lbm-rpm^2)*(rpm-min/ft^3)^2]' /0.5689E-3/
Dh 'AdHead Parameter [lbf-ft/(lbm-rpm^2)*(rpm-min/ft^3)^3]' /-0.1247E-3/
Ae 'AdEff Parameter [1]' /134.8055/
Be 'AdEff Parameter [rpm-s/m^3]' /-148.5468/
Ce 'AdEff Parameter [(rpm-s/m^3)^2]' /125.1013/
De 'AdEff Parameter [(rpm-s/m^3)^3]' /-32.0965/
S_min 'Min Speed [rpm]' /5000/
S_max 'Max Speed [rpm]'/9400/
Surge 'Surge Limit [ft^3/(min-rpm)]' /1.4/
Stonewall 'Stonewall Limit [ft^3/(min-rpm)]' /2.3404255/
L 'Pipe Length [mile]'  /50/
d 'Pipe Diameter [ft]' /3/
Z_st 'Compressability Factor at STP' /1/
conv1 'lb/min to kg/s' /0.00755987/
conv2 'kg^2/s^2 to lb^2/min^2' /17497.204729/
conv3 '[miles-lbm-lbf/(ft^4-min^2)] to [psi^2]' /2.1983696645E-6/
conv4 'psi to MPa' /0.00689476/
conv5 'psi to lbf/ft^2' /144/
conv6 'R to K' /0.55555556/
conv7 'ft to m' /0.3048/
M1 'Big M for Mappa' /1/
M2 'Big M for Z' /1/
dy_vis 'Gas Dynamic Viscosity [Pa s]' /1.0758E-5/
epsilon 'pipeline roughness [m]' /0.05E-3/
;

parameters
arcs_p(i,j) 'set of passive arcs',
arcs_a(i,j) 'set of active arcs',
Vext(i) 'external node V flow [MMSCFD]',
qext(i) 'external node m flow [lbm/min]',
Plow 'lower limit node Pressure [psi]',
Phigh 'upper limit node Pressure [psi]',
T(i) 'node Temperature [K]'
dk(Zs) 'Z reg coef 2 [1]'
ck(Zs) 'Z reg coef 1 [MPa^-1]'
akm(km) 'kappa reg coef 1 [K^-1]'
bkm(km) 'kappa reg coef 2 [MPa^-1]'
ckm(km) 'kappa reg coef 3 [1]'
Ar 'pipieline cross-sectional area [ft^2]'
;

Ar = ((d/2)**2)*pi;

arcs_a('1','2') = 1; arcs_p('2','3') = 1; arcs_a('3','4') = 1; arcs_p('4','5') = 1; arcs_p('5','6') = 1;
arcs_p('5','7') = 1; arcs_a('3','8') = 1; arcs_p('8','9') = 1; arcs_p('9','10') = 1;
arcs_a('7','11') = 1; arcs_p('11','12') = 1; arcs_p('12','13') = 1; arcs_p('12','14') = 1;
arcs_a('14','15') = 1; arcs_p('15','16') = 1; arcs_p('15','17') = 1; arcs_p('17','18') = 1;

Vext('1') = 800; Vext('5') = -100; Vext('9') = -100; Vext('6') = -150; Vext('7') = 500; Vext('10') = -300; Vext('13') = -400; 
Vext('14') = 1000; Vext('16') = -625; Vext('17') = -500; Vext('18') = -125;

qext(i) = Vext(i)*1E6*14.6959/Z_st/R/Tsoil/24/60*conv5;

ck('1') = -25.136118/1000; dk('1') = 0.998755;
*ck('2') = -20.582142/1000; dk('2') = 0.965961;

akm('1') = -0.196274E-3; akm('2') = -0.881420E-3; akm('3') = -2.281592E-3; akm('4') = -0.285367E-3;
bkm('1') = 2.134555E-3; bkm('2') = 20.995274E-3; bkm('3') = 34.205039E-3; bkm('4') = 10.340562E-3;
ckm('1') = 0.276919; ckm('2') = 0.377140; ckm('3') = 0.660269; ckm('4') = 0.273620;

T(i) = Tsoil;

Plow('1') = 600; Plow('2') = 600; Plow('3') = 450; Plow('5') = 450; Plow('6') = 450; Plow('7') = 450; Plow('9') = 450; Plow('4') = 500; Plow('10') = 400; Plow('8') = 550; Plow('11') = 50;  Plow('12') = 50; Plow('13') = 50; Plow('14') = 50; Plow('15') = 100; Plow('16') = 100; Plow('17') = 100; Plow('18') = 100;
Phigh(i) = 800; Phigh('1') = 700;

variables
phi 'fuel cost [10^4 lbf-ft/min]';

positive variables
q(i,j) 'arc flow rate [lbm/min]'
V(i,j) 'volumetric flow rate [ft^3/min]'
Had(i,j) 'Compressor Head [lbf-ft/lbm]'
nad(i,j) 'Compressor Eff [1]'
P(i) 'Node Pressure [psi]'
g(i,j) 'Actual head [lbf-ft/lbm]'
Prat(i,j) 'Compressor Pressure ratio [1]'
zeta(i,j) 'Lumped friction factor and squared flow rate [kg^2/s^2]'
Z_m(i,j) 'Average arc Compressability factor [1]'
P_m(i,j) 'Average arc Pressure [psi]'
z(i) 'Node Compressability Factor [1]'
m(i,j) 'Fuction of isentropic exponent [1]'
VS(i,j) 'ratio of V over S [ft^3/(min-rpm)]'
S(i,j) 'Compressor speed [rpm]'
ff(i,j) 'Friction factor [1]'
Rey(i,j) 'Reynolds number [1]'
CW2(i,j) 'Term in Colebrook-White formula [1]'
;

binary variable
b_Zi(i,j,Zs)
b_Zm(i,j,Zs)
b_m(i,j,km)
;

integer variables
ns(i,j) 'number of compressor units active'
;

equations
FC
mass_bal(i)
piperes(i,j)
RHS_pres(i,j)
Qflow(i,j)
HadCalc(i,j)
Pratio(i,j)
cost(i,j)
Pmean(i,j)
mappa_l(i,j,km)
mappa_u(i,j,km)
mut_m(i,j)
CompD(i,j)
Eff(i,j)
V_S_rat(i,j)
Qlow(i,j)
Qhigh(i,j)
ColeWh(i,j)
ReyEq(i,j)
CWT2_eq(i,j)
Zm_l(i,j,Zs)
Zm_h(i,j,Zs)
mut_Zm(i,j)
Zi_l(i,j,Zs)
Zi_h(i,j,Zs)
mut_Zi(i,j)
;
  
FC.. phi =e= sum((i,j) $(arcs_a(i,j) = 1), g(i,j)*q(i,j))/1e4;

cost(i,j) $(arcs_a(i,j) = 1).. g(i,j)*nad(i,j) =e= Had(i,j);

mass_bal(i).. sum(j $((arcs_a(j,i) = 1) or arcs_p(j,i) = 1), q(j,i)) - sum(j $((arcs_a(i,j) = 1) or (arcs_p(i,j) = 1)), q(i,j)) =e= -qext(i);

piperes(i,j) $(arcs_p(i,j) = 1).. (P(i)**2 - P(j)**2) =e= (L*R*Z_m(i,j)*T(i)/((pi*(d/2)**2)**2*d)*zeta(i,j))*conv3;

RHS_pres(i,j) $(arcs_p(i,j) = 1).. zeta(i,j) =e= ff(i,j)*q(i,j)**2;

ColeWh(i,j) $(arcs_p(i,j) = 1).. 1 =e= -2*log10(epsilon/(d*conv7)/3.71 + CW2(i,j))*sqrt(ff(i,j));

CWT2_eq(i,j) $(arcs_p(i,j) = 1).. CW2(i,j)*Rey(i,j)*sqrt(ff(i,j)) =e= 2.51;

ReyEq(i,j) $(arcs_p(i,j) = 1).. Rey(i,j) =e= d/(Ar*dy_vis)*q(i,j)*conv1/conv7;

Pmean(i,j) $(arcs_p(i,j) = 1).. P_m(i,j) =e= 0.5*(P(i) + P(j));

Zm_l(i,j,Zs) $(arcs_p(i,j) = 1).. Z_m(i,j) =g= ck(Zs)*(P_m(i,j)*conv4) + dk(Zs);

Zm_h(i,j,Zs) $(arcs_p(i,j) = 1).. Z_m(i,j) =l= ck(Zs)*(P_m(i,j)*conv4) + dk(Zs) + M2*(1 - b_Zm(i,j,Zs));

mut_Zm(i,j) $(arcs_p(i,j) = 1).. sum(Zs, b_Zm(i,j,Zs)) =e= 1;

Qflow(i,j) $(arcs_a(i,j) = 1).. V(i,j)*P(i)*ns(i,j)*conv5 =e= z(i)*R*T(i)*q(i,j);

Zi_l(i,j,Zs) $(arcs_a(i,j) = 1).. z(i) =g= ck(Zs)*(P(i)*conv4) + dk(Zs);

Zi_h(i,j,Zs) $(arcs_a(i,j) = 1).. z(i) =l= ck(Zs)*(P(i)*conv4) + dk(Zs) + M2*(1 - b_Zi(i,j,Zs));

mut_Zi(i,j) $(arcs_a(i,j) = 1).. sum(Zs, b_Zi(i,j,Zs)) =e= 1;

HadCalc(i,j) $(arcs_a(i,j) = 1).. Had(i,j)*m(i,j) =e= (z(i)*R*T(i)*(Prat(i,j)**(m(i,j))-1));

mappa_l(i,j,km) $(arcs_a(i,j) = 1).. m(i,j) =g= akm(km)*(T(i)*conv6) + bkm(km)*(P(i)*conv4) + ckm(km);

mappa_u(i,j,km) $(arcs_a(i,j) = 1).. m(i,j) =l= akm(km)*(T(i)*conv6) + bkm(km)*(P(i)*conv4) + ckm(km) + M1*(1 - b_m(i,j,km));

mut_m(i,j)  $(arcs_a(i,j) = 1).. sum(km, b_m(i,j,km)) =e= 1;

Pratio(i,j)  $(arcs_a(i,j) = 1).. Prat(i,j)*P(i) =e= P(j);

CompD(i,j) $(arcs_a(i,j) = 1).. Had(i,j) =e= (Ah + Bh*VS(i,j) + Ch*VS(i,j)**2 + Dh*VS(i,j)**3)*S(i,j)**2;

Eff(i,j) $(arcs_a(i,j) = 1).. nad(i,j) =e= Ae + Be*VS(i,j) + Ce*VS(i,j)**2 + De*VS(i,j)**3;

V_S_rat(i,j) $(arcs_a(i,j) = 1).. VS(i,j)*S(i,j) =e= V(i,j);

Qlow(i,j) $(arcs_a(i,j) = 1).. Surge =l= VS(i,j);

Qhigh(i,j) $(arcs_a(i,j) = 1).. Stonewall =g= VS(i,j);

P.lo(i) = Plow(i);
P.up(i) = Phigh(i);
q.lo(i,j) = 0;
q.l(i,j) $(arcs_p(i,j) = 1 or arcs_a(i,j) = 1) = 25000;
V.lo(i,j) = 0.7E4;
V.up(i,j) = 2.2E4;
S.lo(i,j) = S_min;
S.up(i,j) = S_max;
ns.up(i,j) = 5;
ns.l(i,j) $(arcs_a(i,j) = 1) = 1;

q.scale(i,j) = 1E3;
V.scale(i,j) = 1E3;
Had.scale(i,j) = 1E2;
nad.scale(i,j) = 1;
P.scale(i) = 1E1;
g.scale(i,j) = 1;
Prat.scale(i,j) = 1E-1;
zeta.scale(i,j) = 1E5;
Z_m.scale(i,j) = 1E-2;
P_m.scale(i,j) = 1E1;
z.scale(i) = 1E-2;
m.scale(i,j) = 1E-2;
VS.scale(i,j) = 1E-1;
S.scale(i,j) = 1E3;
ff.scale(i,j) = 1E-3;
Rey.scale(i,j) = 1E6;
CW2.scale(i,j) = 1E-7;
phi.scale = 1E1;

option minlp = scip;
option reslim = 10000000;


model FCMP_PR /
all
/;

option optcr = 1e-5;
option optca = 1e-5;

FCMP_PR.scaleopt = 1;
FCMP_PR.OptFile = 1;

solve FCMP_PR using minlp minimizing phi;

display q.L, Had.l,P.l, z.l, P_m.l, Z_m.l, V.l, m.l, zeta.l, nad.l, ns.l, g.l, ff.l, Rey.l, Prat.l, CW2.l;
