clc; clear all; close all;
format long

conv10 = 0.00298907; %ft-lbf/lbm to kJ/kg
conv20 = 0.000471947; %ft^3/min to m^3/s
Ah = 0.6824E-3; Bh = -0.9002E-3; Ch = 0.5689E-3; Dh = -0.1247E-3;
Ae = 134.8055; Be = -148.5468; Ce = 125.1013; De = -32.0965;
Smin = 5000; Smax = 9400; Surge = 1.4; Stonewall = 22000/9400;

npt = 10000;

%First plot Smin line;

V1i = Smin*Surge;
V1f = Smin*Stonewall;

discV1 = (V1f - V1i)/(npt - 1);

V1 = V1i:discV1:V1f;

S1 = ones(1,length(V1))*Smin;
Had1 = zeros(1,length(V1));
Had12 = zeros(1,length(V1));

for i = 1:length(V1)
    Had1(i) = (Ah*(S1(i)/1000)^3 + Bh*(V1(i)/1000)*(S1(i)/1000)^2 + Ch*(V1(i)/1000)^2*(S1(i)/1000) + Dh*(V1(i)/1000)^3)*1E6/(S1(i))*1E3;
end

figure
hold on
plot(V1*conv20,Had1*conv10,'k')

%Now Plot Smax line

V2i = Smax*Surge;
V2f = Smax*Stonewall;

discV2 = (V2f - V2i)/(npt - 1);

V2 = V2i:discV2:V2f;

S2 = ones(1,length(V2))*Smax;
Had2 = zeros(1,length(V2));

for i = 1:length(V2)
    Had2(i) = (Ah*(S2(i)/1000)^3 + Bh*(V2(i)/1000)*(S2(i)/1000)^2 + Ch*(V2(i)/1000)^2*(S2(i)/1000) + Dh*(V2(i)/1000)^3)*1E6/(S2(i))*1E3;
end

plot(V2*conv20, Had2*conv10,'k')

%Now Plot Surge line

V3i = Smin*Surge;
V3f = Smax*Surge;

discV3 = (V3f - V3i)/(npt - 1);

V3 = V3i:discV3:V3f;

S3 = zeros(1,length(V3));
Had3= zeros(1,length(V3));

for i = 1:length(V3)
    S3(i) = V3(i)/Surge;
    Had3(i) = (Ah*(S3(i)/1000)^3 + Bh*(V3(i)/1000)*(S3(i)/1000)^2 + Ch*(V3(i)/1000)^2*(S3(i)/1000) + Dh*(V3(i)/1000)^3)*1E6/(S3(i))*1E3;
end

plot(V3*conv20,Had3*conv10,'k')

%Now Plot Stonewall line

V4i = Smin*Stonewall;
V4f = Smax*Stonewall;

discV4 = (V4f - V4i)/(npt - 1);

V4 = V4i:discV4:V4f;

S4 = zeros(1,length(V4));
Had4= zeros(1,length(V4));

for i = 1:length(V4)
    S4(i) = V4(i)/Stonewall;
    Had4(i) = (Ah*(S4(i)/1000)^3 + Bh*(V4(i)/1000)*(S4(i)/1000)^2 + Ch*(V4(i)/1000)^2*(S4(i)/1000) + Dh*(V4(i)/1000)^3)*1E6/(S4(i))*1E3;
end

plot(V4*conv20,Had4*conv10,'k')

%Now Check where we are on the domain
R = 85.2;
conv1 = 0.555556; %rankine to kelvin
conv2 = 6894.76; %psi to Pa
conv3 = 144; %ft^2 to in^2

T = 519.67; %rankine

P1 = 369.8013; q = 41489.573;
P2 = 401.037; ns = 3;

Z = Z_Sim(T*conv1,P1*conv2);
kappa = kappa_Sim(T*conv1, P1*conv2);
mappa = (kappa - 1)/kappa;

V_P = Z*R*T*q/(P1*conv3*ns);
Had_P = Z*T*R/(mappa)*((P2/P1)^mappa - 1)/1000;
Output = ['Compressor head is ' num2str(Had_P*1000*conv10,4) 'kJ/kg'];
disp(Output)

plot(V_P*conv20, Had_P*1000*conv10, 'r*')
xlabel('Volumetric Flow Rate(m^3/s)')
ylabel('Adiabatic Head (kJ/kg)')

f_s = @(S_s) Had_P - (Ah*(S_s/1000)^3 + Bh*(V_P/1000)*(S_s/1000)^2 + Ch*(V_P/1000)^2*(S_s/1000) + Dh*(V_P/1000)^3)*1E6/(S_s);

S_P = fzero(f_s, 10000);

nad_P = Ae + Be*V_P/S_P + Ce*(V_P/S_P)^2 + De*(V_P/S_P)^3;
Output = ['Compressor efficiency is ' num2str(nad_P,2) '%'];
disp(Output)


VS = V_P/S_P;

if VS < Surge
    Sdiff = abs(S_P - V_P/Surge)/(V_P/Surge)*100;
    Output = ['Surge limit has been viotlated by ' num2str(Sdiff,2) '%'];
    disp(Output)
end
if VS > Stonewall
    Sdiff = abs(S_P - V_P/Stonewall)/(V_P/Stonewall)*100;
    Output = ['Stonewall limit has been viotlated by ' num2str(Sdiff,2) '%'];
    disp(Output)
end  
if S_P > Smax
    Sdiff = abs(S_P - Smax)/(Smax)*100;
    Output = ['Smax limit has been viotlated by ' num2str(Sdiff,2) '%'];
    disp(Output)
end
if S_P < Smin
    Sdiff = abs(S_P - Smin)/(Smin)*100;
    Output = ['Smin limit has been viotlated by ' num2str(Sdiff,2) '%'];
    disp(Output)
end
